#!/usr/bin/env python
#
#       Author:	Wolter Hellmund
#       Email:	wolterh6@gmail.com
#       --------------------------------------
#       Fileorganizer file operations
#       --------------------------------------
#       Creative Commons

import os, shutil, urllib
import rhythmdb
import tools

metatypes = ('artist','album','title','track')

class Musicfile ():

	def __init__ (self, fileorganizer, db_entry=None):
		self.fo = fileorganizer
		db = self.fo.db
		if db_entry:
			# Track digits magic
			digits = '%s' % ('%0'+str (self.fo.configurator.get_val('digits').get_int ())+'.d')
			self.metadata = {
				metatypes [0]:	db.entry_get (db_entry, rhythmdb.PROP_ARTIST),
				metatypes [1]:	db.entry_get (db_entry, rhythmdb.PROP_ALBUM),
				metatypes [2]:	db.entry_get (db_entry, rhythmdb.PROP_TITLE),
				metatypes [3]:	digits % (db.entry_get (db_entry, rhythmdb.PROP_TRACK_NUMBER))
			}
			self.location = db.entry_get (db_entry, rhythmdb.PROP_LOCATION)
			self.entry = db_entry

	# Returns metadata of the music file
	def get_metadata (self,key):
		for datum in self.metadata:
			if key == datum:
				return self.metadata [datum]

	# Moves the file to a specific location with a specific name
	def relocate (self):
		# Configure path
		targetdir = self.fo.configurator.get_val('folder').get_string ()
		targetdir = tools.datafiller (self, targetdir)
		targetdir = tools.folderize (self.fo.configurator, targetdir)
		# Configure name
		targetname = self.fo.configurator.get_val('file').get_string ()
		targetname = tools.datafiller (self, targetname)
		targetname += os.path.splitext (self.location)[1]
		# Relocate, if necessary
		source = urllib.url2pathname(self.location).replace ('file:///','/')
		destin = os.path.join (self.fo.configurator.get_val('dir').get_string (), \
			targetdir, targetname)
		# Debugging purposes only
		#~ print 'Location: '+urllib.url2pathname (self.location).replace ('file:///','/')
		#~ print 'Destination: '+os.path.join (self.fo.configurator.get_val ('dir'),targetdir,targetname)
		if source == destin:
			print 'No need for file relocation'
		else:
			if os.path.isfile (destin):
				# Copy the existing file to a backup dir
				backup = os.path.join (os.path.dirname (destin),'backup',os.path.basename (destin))
				try:
					os.makedirs (os.path.dirname (backup))
				except:
					pass
				shutil.copy2 (destin, backup)
			
			# Update Rhythmbox database
			db = self.fo.db
			self.location = 'file://'
			self.location += '%s' % destin.replace(' ','%20')
			db.set(self.entry, rhythmdb.PROP_LOCATION, self.location)
			
			print 'Relocating file %s to %s' % (source,destin)
			# Move
			shutil.move (source, destin)
			# Remove empty folders, if any
			sourcedir = os.path.dirname (source)
			if os.listdir (sourcedir) == []:
				print '%s (directory) now empty; removing' % sourcedir
				os.rmdir (sourcedir)
			else:
				print '%s (directory) still contains files; keeping' % sourcedir
